#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>


// macro di utilità per controllare errori in chiamate di libreria
#define CHECK_CALL(X, str, errval )	\
    if ((X)==errval) {			\
	perror(#str);			\
	exit(EXIT_FAILURE);		\
    }
static const char filename[] = "./mat_dump.txt";
static const int MAX_LINE = 256;

int main(int argc, char *argv[]) {

    if (argc != 3) {
	fprintf(stderr, "use: %s <dim-row> <dim-col>\n", argv[0]);
	return EXIT_FAILURE;
    }
    long dimr = strtol(argv[1], NULL, 10);
    long dimc = strtol(argv[2], NULL, 10);

    float *mat = NULL;
    // alloco la matrice come un singolo vettore
    CHECK_CALL(mat = malloc(dimr*dimc*sizeof(float)), "malloc",NULL);
    
    for(long i=0;i<dimr;++i)
	for(long j=0;j<dimc;++j)
	    mat[(i*dimc) + j] = (i+j)/2.0;

    FILE *fp;
    CHECK_CALL(fp = fopen(filename, "w+"), "fopen", NULL);
    CHECK_CALL(fprintf(fp, "%ld\n", dimr), "fprintf", -1);
    CHECK_CALL(fprintf(fp, "%ld\n", dimc), "fprintf", -1);

    for(long i=0;i<dimr;++i)
	for(long j=0;j<dimc;++j)
	    CHECK_CALL(fprintf(fp, "%f\n", mat[i*dimc+j]), "fprintf", -1);
    CHECK_CALL(fclose(fp), "fclose", EOF);

    float *mat2 = NULL;
    CHECK_CALL(mat2 = malloc(dimr*dimc*sizeof(float)), "malloc", NULL);
    CHECK_CALL(fp = fopen(filename, "r"), "fopen", NULL);

    char buf[MAX_LINE];
    CHECK_CALL(fgets(buf, sizeof(buf), fp), "fgets", NULL);
    long tmp; 
    tmp = strtol(buf, NULL, 10);
    assert(tmp == dimr);
    CHECK_CALL(fgets(buf, sizeof(buf), fp), "fgets", NULL);
    tmp = strtol(buf, NULL, 10);
    assert(tmp == dimc);

    for(long i=0;i<dimr;++i)
	for(long j=0;j<dimc;++j) {
	    CHECK_CALL(fgets(buf, sizeof(buf), fp), "fgets", NULL);
	    mat2[i*dimc+j] = strtof(buf, NULL);
	}
    CHECK_CALL(fclose(fp), "fclose", EOF);

    if (memcmp(mat, mat2, dimr*dimc*sizeof(float)) != 0) {
	fprintf(stderr, "Le due matrici non corrispondono\n");
	return EXIT_FAILURE;
    }
    fprintf(stdout, "Le due matrici sono uguali\n");
    free(mat);
    free(mat2);
    return 0;

}




